<?php

namespace Drupal\payment_authnet\Plugin\Payment\MethodConfiguration;

use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\plugin\Plugin\Plugin\PluginSelector\PluginSelectorManagerInterface;
use Drupal\plugin\PluginType\PluginTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\payment\Plugin\Payment\MethodConfiguration\PaymentMethodConfigurationBase;
use Drupal\payment_authnet\Entity\AuthnetProfile;

/**
 * Provides the configuration for the payment_authnet payment method plugin.
 *
 * Plugins extending this class should provide a configuration schema that
 * extends
 * plugin.plugin_configuration.payment_method_configuration.payment_basic.
 *
 * @PaymentMethodConfiguration(
 *   description = @Translation("A payment method enabling payment through Authorize.net."),
 *   id = "payment_authnet",
 *   label = @Translation("Authorize.net")
 * )
 */
class Authnet extends PaymentMethodConfigurationBase implements ContainerFactoryPluginInterface {

  /**
   * The payment status plugin type.
   *
   * @var \Drupal\plugin\PluginType\PluginTypeInterface
   */
  protected $paymentStatusType;

  /**
   * The plugin selector manager.
   *
   * @var \Drupal\plugin\Plugin\Plugin\PluginSelector\PluginSelectorManagerInterface
   */
  protected $pluginSelectorManager;

  /**
   * The list of Authnet Profiles.
   *
   * @var array
   */
  protected $authnetProfiles;

  /**
   * Constructs a new instance.
   *
   * @param mixed[] $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed[] $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translator.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\plugin\Plugin\Plugin\PluginSelector\PluginSelectorManagerInterface $plugin_selector_manager
   *   The plugin selector manager.
   * @param \Drupal\plugin\PluginType\PluginTypeInterface $payment_status_type
   *   The payment status plugin type.
   */
  public function __construct(array $configuration, $plugin_id, array $plugin_definition, TranslationInterface $string_translation, ModuleHandlerInterface $module_handler, PluginSelectorManagerInterface $plugin_selector_manager, PluginTypeInterface $payment_status_type) {
    $configuration += $this->defaultConfiguration();
    parent::__construct($configuration, $plugin_id, $plugin_definition, $string_translation, $module_handler);
    $this->paymentStatusType = $payment_status_type;
    $this->pluginSelectorManager = $plugin_selector_manager;
    $this->authnetProfiles = AuthnetProfile::loadMultiple();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    /** @var \Drupal\plugin\PluginType\PluginTypeManagerInterface $plugin_type_manager */
    $plugin_type_manager = $container->get('plugin.plugin_type_manager');

    return new static($configuration, $plugin_id, $plugin_definition, $container->get('string_translation'), $container->get('module_handler'), $container->get('plugin.manager.plugin.plugin_selector'), $plugin_type_manager->getPluginType('payment_status'));
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'brand_label' => '',
      'execute_status_id' => 'payment_authorized',
      'capture' => TRUE,
      'capture_status_id' => 'payment_success',
      'refund' => TRUE,
      'refund_status_id' => 'payment_refunded',
      'clone_refunded' => FALSE,
      'partial_refund' => TRUE,
      'partial_refund_status_id' => 'payment_partially_refunded',
    ];
  }

  /**
   * Get variable from configuration array.
   *
   * @param string $name
   *   Configuration property name to search for.
   */
  public function __get($name) {
    return isset($this->configuration[$name]) ? $this->configuration[$name] : NULL;
  }

  /**
   * Set value for property in configuration array.
   *
   * @param string $name
   *   Configuration property name to set.
   * @param mixed $value
   *   Configuration property value to set.
   */
  public function __set($name, $value) {
    $this->configuration[$name] = $value;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['plugin_form'] = [
      '#process' => [[$this, 'processBuildConfigurationForm']],
      '#type' => 'container',
    ];

    return $form;
  }

  /**
   * Implements a form API #process callback.
   */
  public function processBuildConfigurationForm(array &$element, FormStateInterface $form_state, array &$form) {
    $element['brand_label'] = [
      '#default_value' => $this->brand_label,
      '#description' => $this->t('The label that payers will see when choosing a payment method. Defaults to the payment method label.'),
      '#title' => $this->t('Brand label'),
      '#type' => 'textfield',
    ];

    $options = ['' => $this->t('- Select a profile -')];
    foreach ($this->authnetProfiles as $id => $authnet_profile) {
      $options[$id] = $authnet_profile->label();
    }
    $profile_ids = array_keys($this->authnetProfiles);

    $element['profile'] = [
      '#default_value' => $this->profile ?: reset($profile_ids),
      '#description' => $this->t('The Authorize.net profile that will be used to connect to Authorize.net.'),
      '#title' => $this->t('Authorize.net profile'),
      '#type' => 'select',
      '#options' => $options,
    ];

    $workflow_group = implode('][', array_merge($element['#parents'], ['workflow']));
    $element['workflow'] = [
      '#type' => 'vertical_tabs',
    ];
    $element['execute'] = [
      '#group' => $workflow_group,
      '#open' => TRUE,
      '#type' => 'details',
      '#title' => $this->t('Execution'),
    ];
    $element['execute']['execute_status'] = $this->getExecutePaymentStatusSelector($form_state)->buildSelectorForm([], $form_state);
    $element['capture'] = [
      '#group' => $workflow_group,
      '#open' => TRUE,
      '#type' => 'details',
      '#title' => $this->t('Capture'),
    ];
    $capture_id = Html::getUniqueId('capture');
    $element['capture']['capture'] = [
      '#id' => $capture_id,
      '#type' => 'checkbox',
      '#title' => $this->t('Add an additional capture step after payments have been executed.'),
      '#default_value' => $this->capture,
    ];
    $element['capture']['plugin_form'] = [
      '#type' => 'container',
      '#states' => [
        'visible' => [
          '#' . $capture_id => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];
    $element['capture']['plugin_form']['capture_status'] = $this->getCapturePaymentStatusSelector($form_state)->buildSelectorForm([], $form_state);
    $refund_id = Html::getUniqueId('refund');
    $element['refund'] = [
      '#group' => $workflow_group,
      '#open' => TRUE,
      '#type' => 'details',
      '#title' => $this->t('Refund'),
    ];
    $element['refund']['refund'] = [
      '#id' => $refund_id,
      '#type' => 'checkbox',
      '#title' => $this->t('Add an additional refund step after payments have been executed.'),
      '#default_value' => $this->refund,
    ];
    $element['refund']['plugin_form'] = [
      '#type' => 'container',
      '#states' => [
        'visible' => [
          '#' . $refund_id => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];
    $element['refund']['plugin_form']['refund_status'] = $this->getRefundPaymentStatusSelector($form_state)->buildSelectorForm([], $form_state);
    $element['refund']['clone_refunded'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Clone payment entity for refund/partial refund operations.'),
      '#default_value' => $this->clone_refunded,
      '#states' => [
        'visible' => [
          '#' . $refund_id => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];

    $partial_refund_id = Html::getUniqueId('partial_refund');
    $element['refund']['partial_refund'] = [
      '#type' => 'container',
      '#states' => [
        'visible' => [
          '#' . $refund_id => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];
    $part_refund_container = &$element['refund']['partial_refund'];
    $part_refund_container['partial_refund'] = [
      '#id' => $partial_refund_id,
      '#type' => 'checkbox',
      '#title' => $this->t('Allow partial refunds.'),
      '#default_value' => $this->partial_refund,
    ];
    $part_refund_container['status'] = $this->getPartialRefundPaymentStatusSelector($form_state)->buildSelectorForm([], $form_state);
    $part_refund_container['status']['container']['#states']['visible']['#' . $partial_refund_id]['checked'] = TRUE;

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->getExecutePaymentStatusSelector($form_state)->validateSelectorForm($form['plugin_form']['execute']['execute_status'], $form_state);
    $this->getCapturePaymentStatusSelector($form_state)->validateSelectorForm($form['plugin_form']['capture']['plugin_form']['capture_status'], $form_state);
    $this->getRefundPaymentStatusSelector($form_state)->validateSelectorForm($form['plugin_form']['refund']['plugin_form']['refund_status'], $form_state);
    $this->getPartialRefundPaymentStatusSelector($form_state)->validateSelectorForm($form['plugin_form']['refund']['partial_refund']['status'], $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->getExecutePaymentStatusSelector($form_state)->submitSelectorForm($form['plugin_form']['execute']['execute_status'], $form_state);
    $this->getCapturePaymentStatusSelector($form_state)->submitSelectorForm($form['plugin_form']['capture']['plugin_form']['capture_status'], $form_state);
    $this->getRefundPaymentStatusSelector($form_state)->submitSelectorForm($form['plugin_form']['refund']['plugin_form']['refund_status'], $form_state);
    $this->getPartialRefundPaymentStatusSelector($form_state)->submitSelectorForm($form['plugin_form']['refund']['partial_refund']['status'], $form_state);

    $parents = $form['plugin_form']['brand_label']['#parents'];
    array_pop($parents);
    $form_state_values = $form_state->getValues();
    $values = NestedArray::getValue($form_state_values, $parents);
    $this->execute_status_id = $this->getExecutePaymentStatusSelector($form_state)->getSelectedPlugin()->getPluginId();
    $this->capture = $values['capture']['capture'];
    $this->capture_status_id = $this->getCapturePaymentStatusSelector($form_state)->getSelectedPlugin()->getPluginId();
    $this->refund = $values['refund']['refund'];
    $this->clone_refunded = $values['refund']['clone_refunded'];
    $this->refund_status_id = $this->getRefundPaymentStatusSelector($form_state)->getSelectedPlugin()->getPluginId();
    $this->partial_refund = $values['refund']['refund'] && $values['refund']['partial_refund']['partial_refund'] ? 1 : 0;
    $this->partial_refund_status_id = $this->getPartialRefundPaymentStatusSelector($form_state)->getSelectedPlugin()->getPluginId();
    $this->brand_label = $values['brand_label'];
    $this->profile = $values['profile'];
  }

  /**
   * Gets the payment status selector for the execute phase.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state object.
   *
   * @return \Drupal\plugin\Plugin\Plugin\PluginSelector\PluginSelectorInterface
   *   Payment status selector for the execute phase.
   */
  protected function getExecutePaymentStatusSelector(FormStateInterface $form_state) {
    $plugin_selector = $this->getPaymentStatusSelector($form_state, 'execute', $this->execute_status_id);
    $plugin_selector->setLabel($this->t('Payment execution status'));
    $plugin_selector->setDescription($this->t('The status to set payments to after being executed by this payment method.'));

    return $plugin_selector;
  }

  /**
   * Gets the payment status selector for the capture phase.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state object.
   *
   * @return \Drupal\plugin\Plugin\Plugin\PluginSelector\PluginSelectorInterface
   *   Payment Status Selector for the capture phase.
   */
  protected function getCapturePaymentStatusSelector(FormStateInterface $form_state) {
    $plugin_selector = $this->getPaymentStatusSelector($form_state, 'capture', $this->capture_status_id);
    $plugin_selector->setLabel($this->t('Payment capture status'));
    $plugin_selector->setDescription($this->t('The status to set payments to after being captured by this payment method.'));

    return $plugin_selector;
  }

  /**
   * Gets the payment status selector for the refund phase.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form State object.
   *
   * @return \Drupal\plugin\Plugin\Plugin\PluginSelector\PluginSelectorInterface
   *   Status selector for the refund phase.
   */
  protected function getRefundPaymentStatusSelector(FormStateInterface $form_state) {
    $plugin_selector = $this->getPaymentStatusSelector($form_state, 'refund', $this->refund_status_id);
    $plugin_selector->setLabel($this->t('Payment refund status'));
    $plugin_selector->setDescription($this->t('The status to set payments to after being refunded by this payment method.'));

    return $plugin_selector;
  }

  /**
   * Gets the payment status selector for the refund phase.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form State object.
   *
   * @return \Drupal\plugin\Plugin\Plugin\PluginSelector\PluginSelectorInterface
   *   Status selector for the refund phase.
   */
  protected function getPartialRefundPaymentStatusSelector(FormStateInterface $form_state) {
    $plugin_selector = $this->getPaymentStatusSelector($form_state, 'partial_refund', $this->partial_refund_status_id);
    $plugin_selector->setLabel($this->t('Payment partial refund status'));
    $plugin_selector->setDescription($this->t('The status to set payments to after being partially refunded by this payment method.'));

    return $plugin_selector;
  }

  /**
   * Gets the payment status selector.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form State object.
   * @param string $type
   *   Selector type.
   * @param string $default_plugin_id
   *   Default plugin ID.
   *
   * @return \Drupal\plugin\Plugin\Plugin\PluginSelector\PluginSelectorInterface
   *   Payment status selector.
   */
  protected function getPaymentStatusSelector(FormStateInterface $form_state, $type, $default_plugin_id) {
    $key = 'payment_status_selector_' . $type;
    if ($form_state->has($key)) {
      $plugin_selector = $form_state->get($key);
    }
    else {
      $plugin_selector = $this->pluginSelectorManager->createInstance('payment_select_list');
      $plugin_selector->setSelectablePluginType($this->paymentStatusType);
      $plugin_selector->setRequired(TRUE);
      $plugin_selector->setCollectPluginConfiguration(FALSE);
      $plugin_selector->setSelectedPlugin($this->paymentStatusType->getPluginManager()->createInstance($default_plugin_id));

      $form_state->set($key, $plugin_selector);
    }

    return $plugin_selector;
  }

}
